"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const server_1 = require("@dash4/server");
const terminal_emulator_1 = require("@dash4/terminal-emulator");
const fs_1 = tslib_1.__importDefault(require("fs"));
const path_1 = tslib_1.__importDefault(require("path"));
const v1_1 = tslib_1.__importDefault(require("uuid/v1"));
tslib_1.__exportStar(require("./jest-commands"), exports);
const processCwd = fs_1.default.realpathSync(process.cwd());
class PluginTerminal extends server_1.Dash4Plugin {
    constructor({ dark = true, width, cmd, cwd, autostart = false, allowedCommands }) {
        super({
            dark,
            width,
            name: 'PluginTerminal',
            lowerCaseName: 'plugin-terminal',
        });
        this._terminalLog = '';
        this._stopProcessingTriggered = false;
        this.connected = () => {
            if (!this.on) {
                return;
            }
            this.on('connected', this.handleConnected);
            this.on('start', this.start);
            this.on('stop', this.stop);
            this.on('clean', this.clean);
            this.on('command', this.appendCommand);
            this.on('command-input', this.appendCommandInput);
        };
        this.handleConnected = () => {
            this.send('connected', this._terminalLog);
            if (this._stopProcessingTriggered) {
                this.send('stopped');
            }
        };
        this.handleTermStopProcessing = () => {
            this._stopProcessingTriggered = true;
            this.send('stopped');
        };
        this.getCommandById = (commandId) => {
            if (!this._allowedCommands || !this._allowedCommands[commandId]) {
                return;
            }
            return this._allowedCommands[commandId];
        };
        this.appendCommand = (commandId) => {
            const command = this.getCommandById(commandId);
            if (!command) {
                return;
            }
            const out = command.terminalOutput || command.keyCode.toString();
            this._terminalLog += out;
            if (!this._term) {
                return;
            }
            this._term.write(out);
        };
        this.appendCommandInput = ({ value, commandId }) => {
            const command = this.getCommandById(commandId);
            if (!command || !command.input || !value || !commandId) {
                return;
            }
            const out = command.input(value);
            this._terminalLog += out;
            if (!this._term) {
                return;
            }
            this._term.write(out);
            setTimeout(() => {
                if (!this._term) {
                    return;
                }
                this._term.write('\r');
            }, 100);
        };
        this.recieveData = (data) => {
            this._terminalLog += data.toString();
            if (!this.send) {
                return;
            }
            this.send('recieve', data.toString());
        };
        this.kill = () => {
            if (!this._term) {
                return;
            }
            this._term.kill();
        };
        this.write = (cmd) => {
            if (!this._term) {
                return;
            }
            this._term.write(cmd);
        };
        this.stop = () => {
            this._terminalLog = '';
            this.kill();
        };
        this.clean = () => {
            this.write('\x1Bc');
            this._terminalLog = '';
        };
        this.start = () => tslib_1.__awaiter(this, void 0, void 0, function* () {
            this.create();
            this._stopProcessingTriggered = false;
            this.write(`${this._cmd}\r`);
        });
        this._cmd = cmd;
        this._cwd = cwd ? path_1.default.join(processCwd, cwd) : processCwd;
        this._autostart = autostart;
        if (allowedCommands) {
            allowedCommands.forEach((allowedCommand) => {
                if (!this._allowedCommands) {
                    this._allowedCommands = {};
                }
                this._allowedCommands[v1_1.default()] = allowedCommand;
            });
        }
        if (this._autostart) {
            this.start();
        }
    }
    create() {
        this._term = terminal_emulator_1.terminalEmulator({
            cwd: this._cwd,
            onData: this.recieveData,
            onStopProcessing: this.handleTermStopProcessing,
        });
    }
    get clientConfig() {
        const allowedCommands = {};
        Object.keys(this._allowedCommands || {}).forEach((commandId) => {
            if (!this._allowedCommands || !this._allowedCommands[commandId]) {
                return;
            }
            allowedCommands[commandId] = {
                keyCode: this._allowedCommands[commandId].keyCode,
                title: this._allowedCommands[commandId].title,
                hasInput: typeof this._allowedCommands[commandId].input === 'function',
            };
        });
        return {
            cmd: this._cmd,
            cwd: this._cwd,
            allowedCommands,
        };
    }
    get clientFiles() {
        return [path_1.default.join(__dirname, '../../dist/plugins/plugin-terminal/main.js')];
    }
}
exports.PluginTerminal = PluginTerminal;
//# sourceMappingURL=index.js.map